;-----------------------------------------------------
;Gestione ciclo funzionamento normale
;Fa parte del mainloop ed  eseguito ogni ms circa (ritardo deli_1ms)

;Effettua le seguenti operazioni:
; 1) Genera un numero casuale per l'effetto tremolo e l'accensione casuale
; 2) Pilota la luminosit di ogni lampada con o senza effetto tremolo
; 3) Verifica se  trascorso il tempo del singolo passo (512 passi per ogni
;    fase, da 20ms a 2.5s circa). Ritorna al mainloop in caso negativo
; 4) F lampeggiare il led Time in funzione del tempo fase selezionato
; 5) Gestisce l'inizio di ciascuna delle 4 fasi (prelevamento dati utente
;    dalla memoria Eeprom)
; 6) Genera il valore di luminosit corrente secondo l'effetto selezionato
; 7) Pilota i 5 rel secodo le impostazioni utente

run_cycle
        call    deli_1ms                ;Ritardo 1 ms con irq attivi

        call    make_random             ;Genera un numero casuale con tempistica
                                        ;variabile da 10 a 100 ms

        call    flicker_drive_lights    ;Genera effetto tremolio sulle luci
                                        ;(se desiderato) e pilota la luminosit
                                        ;delle 5 lampade

        movf    TimeStep,w              ;Continua se trascorso il tempo impostato
        xorwf   Cnt8_3ms,w              ;Cnt8_3ms  incrementata ogni 8.3ms da rb0_int
        btfss   STATUS,Z
        goto    main2
                                        ;*** Eseguito ad ogni passo (da 20ms ... a 2.34s)

        clrf    Cnt8_3ms                ;Azzera contatore 8.3ms
        call    blink_led_time          ;F lampeggiare il led Time

        btfsc   CStepH,0                ;Se CStepH,L multiplo di 512 (0,512,1024,1536)
        goto    rc_1                    ;gestisce l'inizio di ciascuna fase
        movf    CStepL,f                ;prelevando i valori utente da EEprom
        btfss   STATUS,Z
        goto    rc_1
        call    start_fase

rc_1    call    make_lights_value       ;Genera i 5 valori assoluti di luminosit
                                        ;su LtValue(x) secondo l'andamento impostato
                                        ;in LtMode(x) e gestisce stato random on-off

        call    drive_rele              ;Pilota i 5 rel secondo l'andamento prelevato
                                        ;dai settaggi utente in EEprom: sono previsti
                                        ;5 passi per ogni fase.
                                        ;Gestisce stato random on-off

        incfsz  CStepL,f                ;Incrementa contatore passi.
        goto    main2                   ;Azzera se giunto al massimo (2048)
        incf    CStepH,f                ;ogni fase 512 passi
        btfss   CStepH,3                ;4 fasi --> Totale: 2048 passi
        goto    main2
        clrf    CStepH
        clrf    CStepL
        goto    main2                   ;Ritorna al main


;-----------------------------------------------------
;Genera un numero casuale utilizzato per:
; - far tremolare le lampade
; - gestire l'accensione random delle lampade
; - gestire l'accensione random dei rel

;Per aumentare la casualit, la modifica del valore casuale si effettua
;ad intervalli crescenti, da un minimo di 10ms ad un massimo di 100ms circa.
;Questa sub  richiamata ogni 1ms circa nel mainloop

; Input:  Random, DelRnd, CntRnd
; Output: Random, DelRnd, CntRnd

make_random
        decfsz  CntRnd,f        ;Decrementa contatore per ritardo variabile
        return                  ;Esce se <>0

        movf    DelRnd,w        ;Se il ritardo  al massimo
        xorlw   D'100'          ;lo imposta al minimo
        btfss   STATUS,Z
        goto    mrnd_1
        movlw   D'10'
        movwf   DelRnd
        goto    mrnd_2
mrnd_1  incf    DelRnd,f        ;Incrementa la variabile per il ritardo
mrnd_2  movf    DelRnd,w        ;Copia il valore nel contatore CntRnd
        movwf   CntRnd

        btfss   Random,6        ;Generazione numeri casuali in Random ottenuta
        goto    mrnd_3          ;ruotandola a sinistra e facendo entrare in
        btfsc   Random,5        ;carry 1 o 0 secondo la xor dei bit 6 e 5
        goto    mrnd_4
        goto    mrnd_5          ;       Random,6  Random,5   XOR
mrnd_3  btfsc   Random,5        ;          0         0        1
        goto    mrnd_5          ;          0         1        0
                                ;          1         0        0
mrnd_4  bsf     STATUS,C        ;          1         1        1
	goto    mrnd_6
mrnd_5  bcf     STATUS,C
mrnd_6  rlf     Random,f        ;Ruota a sinistra Random e fa entrare il bit Carry
        return


;-----------------------------------------------------
;Genera l'effetto tremolante e pilota la luminosit delle lampade.

;L'effetto tremolo, generato solo sulle lampade abilitate da utente,
;viene effettuato riducendo del 25% la luminosit ad intervalli casuali
;(variabili Light(x)) rispetto al corrispondente valore assoluto
;(variabili LtValue(x)).

; Input:  LtMode(0..4)
; Input:  LtValue(0..4)
; Input:  Random
; Output: Light(0..4)
; Usate:  Count1, Value, Temp

flicker_drive_lights
        clrf    Count1          ;Azzera contatore 5 lampade (0..4)
        movf    Random,w        ;Copia valore casuale su Value
        movwf   Value

fdl_1   movlw   LtValue+0x00    ;Temp = LtValue(x)
        addwf   Count1,w        ;(valore assoluto luminosit) interessata
        movwf   FSR
        movf    INDF,w
        movwf   Temp

        movlw   LtMode+0x00     ;Calcola indirizzo per variabile LtMode()
        addwf   Count1,w
        movwf   FSR

        btfss   INDF,FLICKER    ;Abilitato tremolo? (da variabile LtMode)
        goto    fdl_3           ;No, salta

        btfss   Value,7         ;Se bit casuale =1 riduce la luminosit
        goto    fdl_3           ;della lampada corrispondente del 25%

        bcf     STATUS,C        ;Temp [cio LtValue(x)] = Temp/4
        rrf     Temp,f
        bcf     STATUS,C
        rrf     Temp,f

        movlw   LtValue+0x00    ;Calcola indirizzo per variabile LtValue
        addwf   Count1,w
        movwf   FSR

        movf    INDF,w          ;Se LtValue(x) < 220 effettua ancora Temp/2
        sublw   D'220'          ;Serve per rendere uniforme la variazione
        btfss   STATUS,C        ;del tremolo alle varie luminosit
        goto    fdl_2
        bcf     STATUS,C        ;Temp = Temp/2
        rrf     Temp,f

fdl_2   movf    Temp,w          ;Temp = LtValue(x) - Temp
        subwf   INDF,w
        movwf   Temp

fdl_3   movlw   Light+0x00      ;Imposta luminosit lampada
        addwf   Count1,w        ;Light(x) = Temp
        movwf   FSR
        movf    Temp,w
        movwf   INDF

        rlf     Value,f         ;Ruota valore casuale per prelevare prossimo bit

        incf    Count1,f        ;Incrementa e verifica contatore (0..4)
        movf    Count1,w
        xorlw   0x05
        btfss   STATUS,Z
        goto    fdl_1           ;Salta per prossima variabile
        return


;-----------------------------------------------------
;Gestisce l'inizio di ciascuna 'fase' (una porzione di 1/4 dell'intero ciclo):

; 1) Preleva impostazione tempo da EEprom e, attraverso la tabella rom, carica
;    in TimeStep il valore per ottenere il 'tempo fase' desiderato.

; 2) Preleva impostazioni Andamento+Stati delle 5 lampade e li carica in LtMode (0..5)
;    Se abilitata funzione 'random on-off' genera, dal valore casuale Random,
;    l'abilitazione al funzionamento di ciascuna lampada (relativa alla fase corrente)

; 3) Preleva impostazioni Stati 'random on-off' dei 5 rel e li carica in EnRele
;    Se abilitata genera, dal valore casuale Random, l'abilitazione al funzionamento
;    di ciascun rel (relativa alla fase corrente)

; Input:  CStepH, CStepL                Contatore passi del ciclo
; Input:  Random                        Valore casuale
; Output: TimeStep                      Tempo del singolo passo
; Output: LtMode(x)                     Andamento + Stati luminosit
; Output: EnRele                        Abilitazione on-off rel
; Usate:  Temp, Value, Count1, Count2   Appoggi vari

start_fase
        clrwdt

;Per prova. Accende Rele 1..4 secondo Fase 0..3
;E' necessario non eseguire la sub drive_rele!!!
;        bcf     O_RELE1
;        bcf     O_RELE2
;        bcf     O_RELE3
;        bcf     O_RELE4

;        rrf     CStepH,w
;        andlw   B'00000011'
;        movwf   Value

;        movf    Value,w                 ;Fase1 (0): Rele1 ON
;        btfss   STATUS,Z
;        goto    stf_p0
;        bsf     O_RELE1
;        goto    stf_0
;stf_p0  movf    Value,w                 ;Fase2 (1): Rele2 ON
;        xorlw   0x01
;        btfss   STATUS,Z
;        goto    stf_p1
;        bsf     O_RELE2
;        goto    stf_0
;stf_p1  movf    Value,w                 ;Fase3 (2): Rele3 ON
;        xorlw   0x02
;        btfss   STATUS,Z
;        goto    stf_p2
;        bsf     O_RELE3
;        goto    stf_0
;stf_p2  bsf     O_RELE4                 ;Fase4 (3): Rele3 ON
;stf_0
;Fine per prova

                                        ;1) ****
                                        ;Preleva settaggio tempo (0..15) da locazione
        rrf     CStepH,w                ;Eeprom 0..3 secondo la fase corrente, indicata
        andlw   B'00000011'             ;dai bit 2,1 di CStepH
        call    read_eeprom
        andlw   B'00001111'
        movwf   Temp
        movlw   0x03                    ;Carica valore TimeStep dalla tabella rom
        movwf   PCLATH                  ;secondo l'offeset dell'impostazione letta
        movf    Temp,w                  ;dalla memoria eeprom
        call    TBL_TIMESTEP
        movwf   TimeStep

                                        ;2) ****
                                        ;Preleva Andamento+Stati delle 5 lampade
        movf    Random,w                ;Copia valore random su Value per stato on-off
        movwf   Value                   ;Value,bit0-->LtMode(0) ... Value,bit4-->LtMode(4)
        movlw   LtMode+0x00
        movwf   FSR                     ;Puntatore su prima variabile LtMode (Lampada 1)
        clrf    Count1

stf_1   rrf     CStepH,w                ;Calcola offset per prelievo variabile EEprom
        andlw   B'00000011'             ;(fase+4), dove fase(0..3) = CStepH bit(2:1)
        addlw   0x04
        movwf   Temp
        movf    Count1,w                ;Moltiplica per raggiungere variabile in EEprom
        movwf   Count2                  ;Temp = Temp + Count1 * 4
stf_2   movf    Count2,f                ;Count1: numero lampada (0..4)
        btfsc   STATUS,Z
        goto    stf_3
        movlw   0x04                    ;Temp=Temp+4
        addwf   Temp,f
        decf    Count2,f
        goto    stf_2

stf_3   movf    Temp,w                  ;Carica su LtMode (0..4) il valore da EEprom
        call    read_eeprom             ;all'indirizzo specificato in Temp (4..23)
        andlw   B'00111111'
        movwf   INDF

        btfss   INDF,RND_EN             ;Trasforma il bit RND_EN (on-off random) di
        goto    stf_4                   ;LtMode(x) secondo l'impostazione random on-off
        btfss   Value,0                 ;Value,0 contiene un bit casuale
        goto    stf_5
stf_4   bsf     INDF,ENABLE             ;RND_EN   Value_Random   ENABLE_LIGHT
        goto    stf_6                   ;   0          x               1        sempre on
stf_5   bcf     INDF,ENABLE             ;   1          0               0        off random
stf_6   rrf     Value,f                 ;   1          1               1        on random


;Per test. Abilita / Disabilita gli stati di tutte le 5 lampade
;        bsf     INDF,ENABLE             ;Stato on-off
;        bcf     INDF,FLICKER            ;Stato tremolo
;Fine test

        incf    FSR,f                   ;Individua prossima variabile LtMode(x)
        incf    Count1,f                ;Prossima lampada (0..4), esce se massimo
        movf    Count1,w
        xorlw   0x05
        btfss   STATUS,Z
        goto    stf_1

                                        ;3) ****
                                        ;Preleva da EEprom il valore Random Rel
        rrf     CStepH,w                ;secondo la fase corrente (indirizzo 24..27)
        andlw   B'00000011'             ;Address = fase + 24
        addlw   D'24'
        call    read_eeprom
        movwf   EnRele

        swapf   Random,w                ;Copia valore casuale per random on-off
        movwf   Value                   ;invertendo i 4bit lsb con i 4 msb
                                        ;(diverso dalle luci)

        comf    EnRele,w                ;Trasforma EnRele da impostazione utente
        iorwf   Value,w                 ;random on-off ad abilitazione rel
        andlw   B'00011111'             ;Utilizzati i bit 0..4
        movwf   EnRele                  ;Idem come da tabella precedente
        return


;-----------------------------------------------------
;Genera il valore di luminosit assoluta (senza effetto tremolo) delle 5 lampade,
;sulle variabili LtValue(x), secondo l'andamento impostato in LtMode(x).

;La sub flicker_drive_lights (richiamata nel mainloop) provveder a
;modificare realmente la luminosit delle lampade, copiando i valori assoluti
;LtValue(x) su Light(x).
;Gestisce stato random on-off per ogni lampada (caricato all'inizio di ogni fase)

;Input:  LtMode(x)
;Input:  CStepH, CStepL
;Output: LtValue(x)
;Usate:  Count1, Value, Temp

make_lights_value
        clrf    Count1                  ;Azzera contatore

mlv_1   movlw   LtMode+0x00             ;Calcola indirizzo per LtMode(0..4)
        addwf   Count1,w
        movwf   FSR

        btfss   INDF,ENABLE             ;Se bit0 LtMode(x)=0 (random on-off =0)
        goto    mlv_off                 ;salta per lampada off

        movf    INDF,w                  ;Carica in Value i bit 5:2 di LtMode(x)
        movwf   Value                   ;relativi all'andamento luminosit
        rrf     Value,f
        rrf     Value,w
        andlw   B'00001111'
        movwf   Value

        movf    Value,w                 ;Effetto 0? Luminosit OFF
        btfsc   STATUS,Z
        goto    mlv_off

        xorlw   D'1'                    ;Effetto 1? Luminosit MAX
        btfsc   STATUS,Z
        goto    mlv_max

        movf    Value,w                 ;Effetto 2? Luminosit OFF e poi MAX
        xorlw   D'2'
        btfss   STATUS,Z
        goto    mlv_2
        btfss   CStepH,0
        goto    mlv_off
        goto    mlv_max

mlv_2   movf    Value,w                 ;Effetto 3? Luminosit MAX e poi OFF
        xorlw   D'3'
        btfss   STATUS,Z
        goto    mlv_3
        btfss   CStepH,0
        goto    mlv_max
        goto    mlv_off

mlv_3   movf    Value,w                 ;Effetto 4? Luminosit UP (crescente)
        xorlw   D'4'
        btfsc   STATUS,Z
        goto    mlv_up1

        movf    Value,w                 ;Effetto 5? Luminosit DOWN (decrescente)
        xorlw   D'5'
        btfsc   STATUS,Z
        goto    mlv_dn1

        movf    Value,w                 ;Effetto 6? Luminosit UP e poi DOWN
        xorlw   D'6'
        btfss   STATUS,Z
        goto    mlv_4
        btfss   CStepH,0
        goto    mlv_up2
        goto    mlv_dn2

mlv_4   movf    Value,w                 ;Effetto 7? Luminosit DOWN e poi UP
        xorlw   D'7'
        btfss   STATUS,Z
        goto    mlv_5
        btfss   CStepH,0
        goto    mlv_dn2
        goto    mlv_up2

mlv_5   movf    Value,w                 ;Effetto 8? Luminosit UP e poi MAX
        xorlw   D'8'
        btfss   STATUS,Z
        goto    mlv_6
        btfss   CStepH,0
        goto    mlv_up2
        goto    mlv_max

mlv_6   movf    Value,w                 ;Effetto 9? Luminosit DOWN e poi OFF
        xorlw   D'9'
        btfss   STATUS,Z
        goto    mlv_7
        btfss   CStepH,0
        goto    mlv_dn2
        goto    mlv_off

mlv_7   movf    Value,w                 ;Effetto 10? Luminosit OFF e poi UP
        xorlw   D'10'
        btfss   STATUS,Z
        goto    mlv_8
        btfss   CStepH,0
        goto    mlv_off
        goto    mlv_up2

mlv_8   movf    Value,w                 ;Effetto 11? Luminosit MAX e poi DOWN
        xorlw   D'11'
        btfss   STATUS,Z
        goto    mlv_9
        btfss   CStepH,0
        goto    mlv_max
        goto    mlv_dn2

mlv_9   movf    Value,w                 ;Effetto 12? Luminosit UP e poi OFF
        xorlw   D'12'
        btfss   STATUS,Z
        goto    mlv_10
        btfss   CStepH,0
        goto    mlv_up2
        goto    mlv_off

mlv_10  movf    Value,w                 ;Effetto 13? Luminosit DOWN e poi MAX
        xorlw   D'13'
        btfss   STATUS,Z
        goto    mlv_11
        btfss   CStepH,0
        goto    mlv_dn2
        goto    mlv_max

mlv_11  movf    Value,w                 ;Effetto 14? Luminosit UP e poi UP
        xorlw   D'14'
        btfss   STATUS,Z
        goto    mlv_12
        btfss   CStepH,0
        goto    mlv_up2
        goto    mlv_up2

mlv_12  btfss   CStepH,0                ;Effetto 15? Luminosit DOWN e poi DOWN
        goto    mlv_dn2
        goto    mlv_dn2


mlv_off clrf    Temp                    ;Luminosit OFF (Temp=0)
        goto    mlv_e

mlv_max movlw   0xFF                    ;Luminosit MAX (Temp=255)
        movwf   Temp
        goto    mlv_e

mlv_up1 rrf     CStepL,w                ;Luminosit UP in 512 passi (lento)
        movwf   Temp                    ;Temp = CStepH,L bit8:1
        bcf     Temp,7
        btfsc   CStepH,0
        bsf     Temp,7
        goto    mlv_e

mlv_dn1 rrf     CStepL,w                ;Luminosit DOWN in 512 passi (lento)
        movwf   Temp                    ;Temp = COM (CStepH,L bit8:1)
        bcf     Temp,7
        btfsc   CStepH,0
        bsf     Temp,7
        comf    Temp,f
        goto    mlv_e

mlv_up2 movf    CStepL,w                ;Luminosit UP in 256 passi (veloce)
        movwf   Temp                    ;Temp = CStepL
        goto    mlv_e

mlv_dn2 movf    CStepL,w                ;Luminosit DOWN in 256 passi (veloce)
        movwf   Temp                    ;Temp = COM (CStepL)
        comf    Temp,f


mlv_e   movlw   LtValue+0x00            ;Copia su LtValue(x) il valore di Temp
        addwf   Count1,w
        movwf   FSR
        movf    Temp,w
        movwf   INDF

        incf    Count1,f                ;Incrementa contatore per prossima variabile
        movf    Count1,w                ;LtMode(x) e quindi Light(x)
        xorlw   0x05
        btfss   STATUS,Z                ;Esce se completate le 5 variabili
        goto    mlv_1
        return


;-----------------------------------------------------
;Aggiorna lo stato dei 5 rel ad ogni 'passo rel' secondo le impostazioni
;utente caricate in EEprom.
;Il tempo di ciascuna (delle 4) fasi  stato suddiviso in 5 passi rel.
;Il raggiuntimento di ogni 'passo rel'  indicato dal valore della coppia
;contatori CStepH,CStepL = 0(0), 102(1), 204(2), 306(3), 408(4)
;Nota: CStepH contiene anche l'indicazione della fase (bit2,1) che viene
;scartata leggendo solo il bit0 (lsb) di CStepH!
;Gestisce stato random on-off per ogni rel (caricato all'inizio di ogni fase)

;Input:  CStepH, CStepL
;Input:  EnRele
;Output: O_RELE1 ... O_RELE5
;Usate:  Count1, Count2, Value

drive_rele
        btfsc   CStepH,0                ;Se CStepH,0=0 controlla CStepL (0,102,204)
        goto    drr_1                   ;Se CStepH,0=1 controlla CStepL (306,408)

        clrf    Count1                  ;Passo rel 0 (CStepH,L=0)
        movf    CStepL,w                ;Count1=0
        btfsc   STATUS,Z
        goto    drr_2

        incf    Count1,f                ;Passo rel 1 (CStepH,L=102)
        movf    CStepL,w                ;Count1=1
        xorlw   D'102'
        btfsc   STATUS,Z
        goto    drr_2

        incf    Count1,f                ;Passo rel 2 (CStepH,L=204)
        movf    CStepL,w                ;Count1=2
        xorlw   D'204'
        btfsc   STATUS,Z
        goto    drr_2
        return

drr_1   movlw   0x03                    ;Passo rel 3 (CStepH,L=306)
        movwf   Count1                  ;Count1=3
        movf    CStepL,w
        xorlw   D'50'
        btfsc   STATUS,Z
        goto    drr_2

        incf    Count1,f                ;Passo rel 4 (CStepH,L=408)
        movf    CStepL,w                ;Count1=4
        xorlw   D'152'
        btfss   STATUS,Z
        return
                                        ;Individua l'indirizzo interessato
drr_2   movlw   D'28'                   ;in memoria EEprom
        addwf   Count1,f                ;Count1 = Count1 + 28
        rrf     CStepH,w                ;Effettua Count1 = Count1 + fase * 5
        andlw   B'00000011'             ;dove fase ciclo = 0..3 (CStepH bit2,1)
        movwf   Count2
drr_3   movf    Count2,f
        btfsc   STATUS,Z
        goto    drr_4
        movlw   D'5'
        addwf   Count1,f                ;Count1 = Count1 + 5
        decf    Count2,f
        goto    drr_3

drr_4   movf    Count1,w                ;Legge valore da EEprom. Count1 = address
        call    read_eeprom             ;Ritorna in w il valore letto

        andwf   EnRele,w                ;Maschera il valore letto con le abilitazioni
        movwf   Value                   ;rel (comprende random on-off)

        bcf     O_RELE1                 ;Spegne tutti i rel (per semplicit)
        bcf     O_RELE2
        bcf     O_RELE3
        bcf     O_RELE4
        bcf     O_RELE5
        btfsc   Value,0                 ;Se Value,0 = 1 accende Rele1
        bsf     O_RELE1
        btfsc   Value,1                 ;Se Value,1 = 1 accende Rele2
        bsf     O_RELE2
        btfsc   Value,2                 ;Se Value,2 = 1 accende Rele3
        bsf     O_RELE3
        btfsc   Value,3                 ;Se Value,3 = 1 accende Rele4
        bsf     O_RELE4
        btfsc   Value,4                 ;Se Value,4 = 1 accende Rele5
        bsf     O_RELE5
        return
