;*********************************************************************
;Interrupt handler. Scelta dell'interrupt generato
        ORG     0x04
        movwf   W_Temp          ;Salvataggio registro w
        swapf   STATUS,w        ;Salvataggio registro STATUS
        movwf   St_Temp

        BANK0                   ;Individua banco 0 (nel main  richiamato
                                ;BANK1 durante lettura EEprom
        btfsc   INTCON,INTF     ;Verifica se generato l'Interrupt su RB0
        goto    rb0_int         ;Salta a Interrupt rb0 se INTF=1
        goto    tim_int         ;Altrimenti salta a Timer Interrupt

end_ih  swapf   St_Temp,w       ;Riprisino registro STATUS
        movwf   STATUS
        swapf   W_Temp,f        ;Ripristino registro w
        swapf   W_Temp,w
        retfie

;-----------------------------------------------------
;*** Interrupt Timer ***
;Si genera questo interrupt quando il registro TMR0, continuamente incrementato
;(ogni 2 uS), arriva a 0. E'utilizzato per decrementare da 255 a 0 la
;variabile CntTrc che stabilisce i 'passi' del ritardo di conduzione del
;triac rispetto al passaggio sullo zero della tensione sinosoidale.

;Inoltre pone a 1 logico le 4 uscite delle 5 lampade (collegate ai triac)
;quando VALORE_LAMPADA>=CntTrc.
;Il triac rimarr in conduzione fino al prossimo passaggio sullo zero.

;Poich, con bassa luminosit, una modifica del suo valore appare a vista 
;in un incremento o decremento non proprio graduale (a gradini),
;si preferisce rendere lo scatto della parzializzazione pi fitto
;durante la parte discendente della tensione sinosoidale.
;Quindi si esegue tim_int utilizzando due tempi differenti
; - CntTrc (255...161) --> luminosit elevata, scatto parzializz. maggiore 54us
; - CntTrc (160...0)   --> luminosit minima, scatto parzializz. minore 25us

tim_int                         ;Ricarica TMR0 per eseguire il prossimo
        movf    CntTrc,w        ;tim_int secondo il valore di CntTrc
        sublw   D'160'          ;w=160-CntTrc (setta flag carry se CntTrc<=160)
        movlw   D'246'          ;ricarica TMR0 per esecuzione prossimo tim_int
        btfss   STATUS,C        ;(256-246) * 2us = 20us + 5us(scelta) ==> 25us
        movlw   D'232'          ;(256-232) * 2us = 48us + 5us(scelta) ==> 54us
        movwf   TMR0            ;25us * 160 volte = 4000us
                                ;54us * 95 volte =  5130us
                                ;Totale    --->     9130us  (circa 10ms)

        movf    CntTrc,w        ;Lampada 1
        subwf   Light+0x00,w    ;w = Light+0x00 - CntTrc
        btfsc   STATUS,C        ;Se Light+0x00<CntTrc salta prossima istruzione
        bsf     O_LIGHT1        ;Porta in conduzione il triac che pilota la
                                ;lampada 1 in ritardo rispetto allo 0

        movf    CntTrc,w        ;Lampada 2
        subwf   Light+0x01,w
        btfsc   STATUS,C
        bsf     O_LIGHT2

        movf    CntTrc,w        ;Lampada 3
        subwf   Light+0x02,w
        btfsc   STATUS,C
        bsf     O_LIGHT3

        movf    CntTrc,w        ;Lampada 4
        subwf   Light+0x03,w
        btfsc   STATUS,C
        bsf     O_LIGHT4

        movf    CntTrc,w        ;Lampada 5
        subwf   Light+0x04,w
        btfsc   STATUS,C
        bsf     O_LIGHT5

        movf    CntTrc,f        ;Se CntTrc<>0 decrementa CntTrc
        btfss   STATUS,Z        ;Salta prossima se CntTrc=0
        decf    CntTrc,f        ;Decrementa CntTrc

        bcf     INTCON,T0IF     ;Riabilita interrupt su TMR0
        goto    end_ih


;-----------------------------------------------------
;*** Interrupt RB0 ***
;Si genera questo interrupt quando l'ingresso RB0 passa da 0 a 1 in presenza
;dello 'zero' della tensione sinosoidale. E' utilizzato per porre a 0 l'uscita
;dei 5 triac associati alle lampade e ad inizializzare la variabile CntTrc.
;Inoltre:
; - Incrementa il contatore Cnt10ms ogni 10 mS (100Hz) fino al raggiungimento
;   del valore TimeStep, che definisce il tempo del singolo passo.
; - Azzera il contatore per non entrare nella routine di comunicazione.
;   Quando l'utente sposta l'apposito deviatore nella modalit comunicazione,
;   questa routine non  pi eseguita e il mainloop entra nella modalit
;   comunicazione. E' necessario un reset per ritornare al funzionamento normale

rb0_int bcf     O_LIGHT1        ;Pone a 0 le uscite dei triac per le 5 Lampade
        bcf     O_LIGHT2
        bcf     O_LIGHT3
        bcf     O_LIGHT4
        bcf     O_LIGHT5

        movlw   D'255'          ;Inizializza CntTrc a 255
        movwf   CntTrc

        movf    Cnt10ms,w       ;Incrementa Cnt10ms se diverso da TimeStep
        xorwf   TimeStep,w
        btfss   STATUS,Z
        incf    Cnt10ms,f

	movlw   D'190'		;Attesa 380uS per non far eseguire tim_int
	movwf   CntZero		;quando la tensione sinosoidale non  ancora
rb0_1   nop                     ;a zero.
        decfsz  CntZero,f
        goto    rb0_1

        clrf    CComm           ;Azzera contatore CComm per non far andare
                                ;il mainloop nella routine di comunicazione
        bcf     INTCON,INTF     ;Riabilita interrupt su RB0
        goto    end_ih


