;-----------------------------------------------------
;Gestione Men principale

;Accende i led associati alle uscite LIGHT1...LIGHT5 per indicare
;all'utente la possibilit di accedere ad uno dei 5 men premendo
;il pulsante PMENU
;Richiama i men 1..4 (impostazione tempi) o il men 5 (test uscite)
;se l'utente preme il pulsante durante il tempo in cui  acceso il
;led corrispondente. Alla fine ritorna al ciclo di funzionamento normale
; Output: O_LIGHT1...5
; Output: Menu
; Input:  I_PMENU
; Usate:  CPbutt

choose_menu
        call    out_irq_off     ;Disattiva irq e spegne tutte le uscite
        call    del_1s
        movlw   0x01            ;Men 1
        movwf   Menu
cmu_1   call    out_irq_off
        bcf     O_LTIME         ;Led Time on: segnalazione men principale

        movf    Menu,w          ;Accende le uscite luci 1..5 per indicare
        xorlw   0x01            ;il menu da scegliere
        btfss   STATUS,Z        ;Indica Menu1
        goto    cmu_2
        bsf     O_LIGHT1
        goto    cmu_6
cmu_2   movf    Menu,w          ;Indica Menu2
        xorlw   0x02
        btfss   STATUS,Z
        goto    cmu_3
        bsf     O_LIGHT2
        goto    cmu_6
cmu_3   movf    Menu,w          ;Indica Menu3
        xorlw   0x03
        btfss   STATUS,Z
        goto    cmu_4
        bsf     O_LIGHT3
        goto    cmu_6
cmu_4   movf    Menu,w          ;Indica Menu4
        xorlw   0x04
        btfss   STATUS,Z
        goto    cmu_5
        bsf     O_LIGHT4
        goto    cmu_6
cmu_5   bsf     O_LIGHT5        ;Indica Menu5

cmu_6   movlw   D'20'           ;Attende che l'utente preme il pulsante entro 2
        movwf   CPbutt          ;secondi per accedere al menu corrente
cmu_7   call    del_100ms
        btfss   I_PMENU
        goto    cmu_8
        decfsz  CPbutt,f
        goto    cmu_7
        goto    cmu_11          ;Se trascorsi 2s, incrementa per prossimo menu

cmu_8   call    out_irq_off     ;Entra in uno dei 5 menu selezionati
        call    led_begin_menu
        movf    Menu,w
        xorlw   0x05
        btfss   STATUS,Z
        goto    cmu_9
        call    menu5           ;Richiama il menu 5 per Test Uscite
        goto    cmu_10
cmu_9   call    menu1234        ;Richiama i menu 1..4 per settaggio tempi
cmu_10  call    out_irq_off
        call    led_end_menu

cmu_11  incf    Menu,f          ;Incrementa per prossimo menu
        movf    Menu,w
        xorlw   0x06
        btfss   STATUS,Z        ;Giunto al massimo?
        goto    cmu_1           ;No, ripete per prossimo menu
        call    out_irq_off     ;Si, esce e riavvia il programma
        call    del_1s
        goto    start


;-----------------------------------------------------
;Gestione Menu 0 (Caricamento valori di default in eeprom)
;Imposta tempi, andamento luci e rel con "default presepe"
;Si accede a questo menu solo premendo il pulsante per 2 secondi dopo il reset
; Input:  TBL_EEDEFAULT         Tabella rom
; Output: Eeprom (0..47)        Locazioni eeprom utente
; Usate:  Count1

menu0   call    led_begin_menu

        clrf    Count1          ;Azzera contatore
m0_1	movlw	0x03  		;Tabella in pagina 3
	movwf	PCLATH
	movf    Count1,w
	call	TBL_EEDEFAULT	;Preleva il valore dalla tabella

        movwf	EEDATA          ;Dato da scrivere su eeprom
        movf    Count1,w
        movwf   EEADR           ;Indirizzo eeprom
        call    write_eeprom

        incf    Count1,f        ;Prossima locazione (0..47)
        movf    Count1,w
        xorlw   D'48'
        btfss   STATUS,Z
        goto    m0_1

        call    del_1s          ;Attesa 1 secondo
        call    led_end_menu
        return

;-----------------------------------------------------
;Gestione Menu 1..4 (Impostazione tempi per ciascuna delle 4 fasi)
;Legge il valore da EEprom (locazioni 0..3, valore 0..15), lo visualizza
;sui 10 led delle uscite e ne permette la modifica (su incremento)
;da parte dell'utente premendo il pulsante PTime.
;Infine, quando l'utente non preme il pulsante per pi di 5s,
;salva il valore su EEprom nella stessa locazione di partenza ed esce.

;Esempi di segnalazione vu-meter sui 10 led delle uscite
; Value=0       Led=X---------          (1)     minimo
; Value=3       Led=XXXX------          (4)
; Value=9       Led=XXXXXXXXXX          (10)
; Value=11      Led=XX-------X          (12)
; Value=15      Led=XXXXXX---X          (16)    massimo

; Input:  Menu
; Output: O_LIGHT1...5, O_RELE1...5
; Input:  I_PMENU
; Output: EEprom (0..3)
; Usate:  Value, Temp

menu1234
        decf    Menu,w          ;Calcola l'indirizzo EEprom per valore tempo
        call    read_eeprom     ;Ritorna in w il valore della EEprom
        andlw   B'00001111'     ;e lo maschera con i 4 bit LSB (limita a 0..15)
        movwf   Value

                                ;Genera segnalazione vu-meter 0..15 sui 10 led
                                ;di uscita secondo Value
m14_0   call    out_irq_off
        bsf     O_LIGHT1        ;Accende sempre il Led 1 (O_LIGHT1)

        movf    Value,w         ;Temp=Value
        movwf   Temp
        movlw   D'10'           ;Se Temp>9, accende led 10
        subwf   Temp,w          ;(w=Temp-10)
        btfss   STATUS,C
        goto    m14_1
        bsf     O_RELE5         ;Accende Led 10 (O_RELE5)
        movlw   D'10'           ;Effettua Temp=Temp-10
        subwf   Temp,f

m14_1   movf    Temp,w          ;Se Temp<>0 Accende Led 2 (O_LIGHT2)
        btfsc   STATUS,Z
        goto    m14_2
        bsf     O_LIGHT2
        decf    Temp,f          ;Se Temp<>0 Accende Led 3 (O_LIGHT3)
        btfsc   STATUS,Z
        goto    m14_2
        bsf     O_LIGHT3
        decf    Temp,f          ;Se Temp<>0 Accende Led 4 (O_LIGHT4)
        btfsc   STATUS,Z
        goto    m14_2
        bsf     O_LIGHT4
        decf    Temp,f          ;Se Temp<>0 Accende Led 5 (O_LIGHT5)
        btfsc   STATUS,Z
        goto    m14_2
        bsf     O_LIGHT5
        decf    Temp,f          ;Se Temp<>0 Accende Led 6 (O_RELE1)
        btfsc   STATUS,Z
        goto    m14_2
        bsf     O_RELE1
        decf    Temp,f          ;Se Temp<>0 Accende Led 7 (O_RELE2)
        btfsc   STATUS,Z
        goto    m14_2
        bsf     O_RELE2
        decf    Temp,f          ;Se Temp<>0 Accende Led 8 (O_RELE3)
        btfsc   STATUS,Z
        goto    m14_2
        bsf     O_RELE3
        decf    Temp,f          ;Se Temp<>0 Accende Led 9 (O_RELE4)
        btfsc   STATUS,Z
        goto    m14_2
        bsf     O_RELE4
        decf    Temp,f          ;Se Temp<>0 Accende Led 10 (O_RELE5)
        btfsc   STATUS,Z
        goto    m14_2
        bsf     O_RELE5

m14_2   movlw   D'10'           ;Aspetta che l'utente preme il pulsante per
        movwf   CPbutt          ;incrementare il valore (0..15)
m14_3   call    del_500ms       ;Si esce (e si salva) se non si preme per 5sec
        btfss   I_PMENU
        goto    m14_4
        decfsz  CPbutt,f
        goto    m14_3
        goto    m14_5

m14_4   incf    Value,f         ;Pulsante premuto. Incrementa valore fino a 15
        btfsc   Value,4
        clrf    Value
        goto    m14_0

m14_5   decf    Menu,w          ;Salva Value su EEprom alla locazione Menu-1
        movwf   EEADR
        movf    Value,w
        movwf   EEDATA
        call    write_eeprom
        return


;-----------------------------------------------------
;Gestione Menu 5 (Test funzionamento 10 uscite)
;Incremento e decremento luminosit delle 5 lampade in sequenza
;Accensione dei 5 rel in sequenza
;Il ciclo viene eseguito 2 volte, dopodich si ritorna al ciclo normale
; Output: Light+0x00 ... Light+0x04
; Output: O_RELE1 ... O_RELE5
; Usate:  Count1

menu5   clrf    Light+0x00      ;Inizializza luminosit lampade a 0
        clrf    Light+0x01
        clrf    Light+0x02
        clrf    Light+0x03
        clrf    Light+0x04

        movlw   0x02            ;Contatore 2 cicli di test funzionamento
        movwf   Count1
m5_1    bsf     INTCON,GIE      ;Abilita interrupt
        movlw   Light+0x00      ;Imposta il puntatore sulla 1^ variabile
        movwf   FSR             ;che regola la luminosit

m5_2    incf    INDF,f          ;Incrementa luminosit fino a 255 (max)
        call    deli_10ms       ;Tempo = 10ms * 255 = 2.5s
        movf    INDF,w
        xorlw   D'255'
        btfss   STATUS,Z
        goto    m5_2

m5_3    decf    INDF,f          ;Decrementa luminosit fino a 0 (min)
        call    deli_10ms       ;Tempo = 10ms * 255 = 2.5s
        movf    INDF,w
        btfss   STATUS,Z
        goto    m5_3

        incf    FSR,f           ;Incrementa puntatore e verifica se massimo
        movf    FSR,w
        xorlw   Light+0x05
        btfss   STATUS,Z
        goto    m5_2            ;Procede con prossima variabile (lampada)

        call    out_irq_off     ;Disattiva irq e pone uscite a 0

        bsf     O_RELE1         ;Accensione dei 5 uscite rel in sequenza
        call    del_5s
        bcf     O_RELE1
        bsf     O_RELE2
        call    del_5s
        bcf     O_RELE2
        bsf     O_RELE3
        call    del_5s
        bcf     O_RELE3
        bsf     O_RELE4
        call    del_5s
        bcf     O_RELE4
        bsf     O_RELE5
        call    del_5s
        bcf     O_RELE5

        decfsz  Count1,f        ;Ripete il ciclo per la seconda volta
        goto    m5_1
        return                  ;Esce


;-----------------------------------------------------
;Genera la segnalazione sul led Time per Inizio e Fine menu
; - Inizio: Attesa 1 secondo - lampeggio 2 volte - Attesa 1 secondo
; - Fine:   Attesa 1 secondo - lampeggio 5 volte - Attesa 1 secondo

; Output: O_LTIME
; Usate:  Count1

led_begin_menu                  ;Segnalazione inizio menu
        movlw   D'2'
        goto    lbem_1

led_end_menu                    ;Segnalazione fine menu
        movlw   D'5'
lbem_1  movwf   Count1

        bsf     O_LTIME
        call    del_1s          ;Attesa 1 secondo
lbem_2  bcf     O_LTIME         ;Led Time acceso per 100ms
        call    del_100ms
        bsf     O_LTIME         ;Led Time spento per 100ms
        call    del_100ms
        decfsz  Count1,f
        goto    lbem_2
        call    del_1s          ;Attesa 1 secondo
        return

