;************************************************************************
; PRESEPE5.ASM Rev.5
; Varia la luce di 5 lampadine a filamento e pilota 5 uscite a rel
; Le variazioni di luminosit, le accensioni e i tempi del ciclo 
; (suddivisi in 4 fasi temporali) possono essere programmate a piacimento
; attraverso un software che dialoga sulla porta seriale
; Maggiori informazioni nel file PRESEPE5.PDF
;************************************************************************

        PROCESSOR       16F84                   ;Tipo di processore
        INCLUDE         "PIC16F84.INC"          ;inclusione File .inc
        __CONFIG        3FF6H                   ;Nessuna protezione codice
                                                ;Watch-dog abilitato
                                                ;Power Up abilitato
                                                ;Oscillatore tipo HS (8 MHz)

;*********************************************************************
;Definizione Macro per selezionare i 2 banchi di memoria Ram
BANK0	MACRO
	bcf     STATUS,RP0
	ENDM
BANK1	MACRO
	bsf     STATUS,RP0
	ENDM

;************************************************************************
;Dichiarazione costanti I/O
#DEFINE O_LIGHT1 PORTB,7        ;Uscita Lampada 1 (alba)
#DEFINE O_LIGHT2 PORTB,6        ;Uscita Lampada 2 (giorno)
#DEFINE O_LIGHT3 PORTB,5        ;Uscita Lampada 3 (tramonto)
#DEFINE O_LIGHT4 PORTB,4        ;Uscita Lampada 4 (notte)
#DEFINE O_LIGHT5 PORTB,3        ;Uscita Lampada 5 (stelle)
#DEFINE O_RELE1  PORTB,2        ;Uscita Rel 1    (zona 1)
#DEFINE O_RELE2  PORTB,1        ;Uscita Rel 2    (zona 2)
#DEFINE O_RELE3  PORTA,4        ;Uscita Rel 3    (zona 3)
#DEFINE O_RELE4  PORTA,3        ;Uscita Rel 4    (zona 4)
#DEFINE O_RELE5  PORTA,2        ;Uscita Rel 5    (zona 5)

#DEFINE I_PMENU  PORTA,1        ;Ingresso pulsante MENU
#DEFINE O_LTIME  PORTA,0        ;Uscita led TIME
#DEFINE I_100HZ  PORTB,0        ;Ingresso sincronismo di rete 100Hz

#DEFINE I_RX     PORTB,0        ;Ingresso seriale dati (duplicato)
#DEFINE O_TX     PORTA,2        ;Uscita seriale dati   (duplicato)


;************************************************************************
;Dichiarazione costanti
MAXEEA  EQU     D'48'           ;Locazioni EEprom utilizzate per
                                ;impostazioni utente (0..47)

RND_EN  EQU     0x00            ;Bit Stato 'on-off Random' Luci in var LtMode(x)
ENABLE  EQU     0x00            ;Bit Stato Abilitazione Luci in var LtMode(x)
                                ;(lo stesso bit che assume 2 diverse funzioni)
FLICKER EQU     0x01            ;Bit Stato Tremolo Luci in var LtMode(x)


;************************************************************************
;Dichiarazione variabili Ram
        ORG     0x0C
Light   RES     0x05            ;Luminosit lampade 1..5 (0=off..255=max)
                                ;per comandare in ritardo i triac

LtValue RES     0x05            ;Valore assoluto luminosit lampade (0=off..255=max)
                                ;(senza modifiche dovute all'effetto tremolo)

LtMode  RES     0x05            ;Andamento + stati lampade 1..5
                                ;bit 5,4,3,2 = andamento (16 effetti)
                                ;bit 1 = Stato tremolo
                                ;bit 0 = Stato random on-off / Abilitazione

EnRele  RES     0x01            ;Abilitazione on-off per 5 rel (bit 0..4)

CStepH  RES     0x01            ;Contatore passi del ciclo giornata
CStepL  RES     0x01            ;(512 passi x 4 fasi = 2048 passi max)

TimeStep RES    0x01            ;Tempo del singolo passo (moltiplicare *10mS)
Cnt10ms  RES    0x01            ;Contatore incrementato ogni 10mS sul fronte
                                ;positivo di RB0 (Frequenza rete 50x2=100Hz)

CntTrc  RES     0x01            ;Contatore per ritardo conduzione triac
CntZero RES     0x01	        ;Ritardo per attesa zero-crossing reale

Random  RES     0x01            ;Generatore numeri casuali per tremolo e random on-off
DelRnd  RES     0x01            ;Tempo (10..100 ms) di ritardo variabile per
                                ;generatore numeri casuali
CntRnd  RES     0x01            ;Contatore per ritardo variabile numeri casuali

Menu    RES     0x01            ;Accesso ai menu utente 1..5

Value   RES     0x01            ;Appoggio temporaneo valore per vari usi
Temp    RES     0x01            ;Appoggio temporaneo per vari usi

CPbutt  RES     0x01            ;Contatore per pressione/rilascio pulsante
CComm   RES     0x01            ;Contatore per accedere alla routine comunicazione

RegTxRx RES     0x01            ;Carattere da trasmettere/ricevuto da seriale
CBit    RES     0x01            ;Contatore bit trasmessi/ricevuti
CDelBit RES     0x01            ;Contatore ritardo per singolo bit

Count1  RES     0x01            ;Contatori per vari utilizzi
Count2  RES     0x01

CntDel1 RES     0x01            ;Contatori per ritardi software
CntDel2 RES     0x01
CntDel3 RES     0x01

W_Temp  RES     0x01            ;Salvataggio registro W durante Interrupt
St_Temp RES     0x01            ;Salvataggio registro STATUS durante Interrupt


;************************************************************************
;Inizio memoria programma (locazione 00)
        ORG     0x00
        goto    start


;************************************************************************
;Inclusione routine per gestioni varie
        INCLUDE "IRQS.INC"      ;Interrupt
        INCLUDE "SUBS.INC"      ;Subroutine varie
        INCLUDE "CYCLE.INC"     ;Gestione ciclo funzionamento normale
        INCLUDE "MENU.INC"      ;Gestione men utente
        INCLUDE "COMM.INC"      ;Comunicazione seriale


;************************************************************************
;Programma PRINCIPALE

;Configurazione porte I/O (vedi #DEFINE all'inizio) e settaggi vari (1=IN 0=OUT)
start   BANK1                   ;Ram Banco 1
        movlw   B'11100010'     ;Configurazione Porta A
        movwf   TRISA
        movlw   B'00000001'     ;Configurazione Porta B
        movwf   TRISB
        movlw   B'01000001'     ;bit7: Attiva circuiteria weak-up ingressi
        movwf   OPTION_REG      ;bit6: Attiva interrupt RB0 su fronte salita
                                ;bit5: Ingresso prescaler a fOsc/4 (1 MHz)
                                ;bit3: Ingresso registro TMR0 a Prescaler
                                ;bit2,1,0: Divisione prescaler 1:4
                                ;Frequenza uscita prescaler = 500000 Hz
        BANK0

        movlw   B'00110000'     ;Abilita Interrupt su Timer e su porta RB0
        movwf   INTCON          ;Manca l'abilitazione globale (bit 7)

        call    out_irq_off     ;Disattiva tutte le uscite

        call    del_1s          ;Se premuto pulsante dopo reset per t>2sec
        btfsc   I_PMENU         ;carica valori di default in eeprom  (menu 0)
        goto    main01
        call    del_1s
        btfsc   I_PMENU
        goto    main01
        call    menu0
        goto    start           ;Riavvia il programma


                                ;*** Inizializza variabili per ciclo normale (puntatore)

main01  movlw   Light+0x00      ;Luci spente Light(x)=0
        movwf   FSR             ;Valori assoluti luminosit lampade =0
        movlw   D'22'           ;Andamento+Stati luci non selezionato LtMode(x)=0
        movwf   Count1          ;Azzera contatori per passo ciclo (CStepH,L=0)
main02  clrf    INDF            ;Azzera contatori per tempo singolo passo
        incf    FSR,f           ;(TimeStep=0 e Cnt10ms=0)
        decfsz  Count1,f        ;Azzera contatore per inizio comunicazione (CComm=0)
        goto    main02

        clrf    Random          ;Inizializza valore casuale Random=0
        movlw   D'10'           ;Inizializza ritardo per generatore
        movwf   DelRnd          ;di numeri casuali al minimo
        movwf   CntRnd

        bsf     INTCON,GIE      ;Abilita gli interrupt precedentemente selezionati

                                ;*** Loop principale
mainloop
        btfsc   I_PMENU         ;Se premuto PMenu per t>2s entra nella
        goto    main1           ;gestione per scelta men utente
        bsf     O_LTIME         ;Spegne il led time
        call    deli_100ms
        decfsz  CPbutt,f
        goto    mainloop
        goto    choose_menu

main1   movlw   D'20'           ;Pulsante PMenu non premuto
        movwf   CPbutt          ;Esegue il ciclo di funzionamento normale
        goto    run_cycle

main2   incf    CComm,f         ;Verifica se si vuole entrare nella gestione
        movf    CComm,w         ;comunicazione. CComm  continuamente azzerata
        xorlw   D'250'          ;dall'irq rb0 durante il funzionamento normale.
        btfss   STATUS,Z        ;Raggiunge 250 quando irq rb0 non viene pi
        goto    mainloop        ;eseguito perch l'utente ha spostato l'apposito
        goto    make_comm       ;deviatore per la comunicazione.


;************************************************************************
;Inclusione Tabelle Rom
        INCLUDE "TABLES.INC"    ;Tabelle rom


        END                     ;Fine programma

