;-----------------------------------------------------
;Inverte lo stato del led Time per generare il lampeggio
; Output: O_LTIME

blink_led_time
        btfss   O_LTIME
        goto    blt_1
        bcf     O_LTIME
        return
blt_1   bsf     O_LTIME
        return


;-----------------------------------------------------
;Disattiva interrupt e spegne tutte le uscite
; Output: O_LIGHT1 ... O_LIGHT5
; Output: O_RELE1 ... O_RELE5

out_irq_off
        bcf     INTCON,GIE      ;Disattiva irq

        bcf     O_LIGHT1        ;Pone a 0 le 5 uscite lampade
        bcf     O_LIGHT2
        bcf     O_LIGHT3
        bcf     O_LIGHT4
        bcf     O_LIGHT5

        bcf     O_RELE1         ;Pone a 0 le 5 uscite rel (on-off)
        bcf     O_RELE2
        bcf     O_RELE3
        bcf     O_RELE4
        bcf     O_RELE5
        bsf     O_LTIME         ;Spegne led Time (invertito)
        return


;-----------------------------------------------------
;Legge dalla memoria EEprom il dato all'indirizzo specificato in  w
; Input:  w     Indirizzo di lettura dalla EEprom
; Output: w     Dato letto dalla EEprom

read_eeprom
        BANK1
        movwf   EEADR           ;Copia w (indirizzo) in EEADR
        bsf     EECON1,RD
        movf    EEDATA,w        ;Ritorna il dato letto in w
        BANK0
        return


;-----------------------------------------------------
;Scrive sulla memoria EEprom il dato all'indirizzo specificato
; Input:  EEADR         Indirizzo di scrittura
; Input:  EEDATA        Dato da scrivere
; Output: EEprom

write_eeprom
;        BANK1                  ;Gi in BANK1
        bcf     INTCON,GIE      ;Disabilita tutti gli interrupt
        bsf     EECON1,WREN     ;Sequenza obbligata
        movlw   0x55
        movwf   EECON2
        movlw   0xAA
        movwf   EECON2
        bsf     EECON1,WR       ;Start scrittura
wee_1   clrwdt                  ;Aspetta finch  completata la scrittura
        btfsc   EECON1,WR
        goto    wee_1
;        bcf     EECON1,WR       ;Inutile: viene resettato da hardware
        BANK0
        return


;-----------------------------------------------------
;Effettua ritardi da 100mS a 2 secondi (senza interrupt)
;Con gli interrupt abilitati durano circa 3 volte di pi

; Usate: CntDel1, CntDel2, CntDel3

del_5s                          ;Ritardo 5 secondi (50 * 100 ms)
        movlw   D'50'
        goto    dels_1

del_1s                          ;Ritardo 1 secondo (10 * 100 ms)
        movlw   D'10'
        goto    dels_1

del_500ms                       ;Ritardo 0,5 secondi (5 * 100 ms)
        movlw   D'5'
        goto    dels_1

del_100ms                       ;Ritardo 0,1 secondi (1 * 100 ms)
        movlw   D'1'
dels_1  movwf   CntDel3

dels_2  movlw   D'196'          ;Ritardo [(4*255)+3]*196=200508/2 = 100254us
        movwf   CntDel2
        clrf    CntDel1
dels_3  clrwdt                  ;Ciclo 1
        decfsz  CntDel1,f
        goto    dels_3
        decfsz  CntDel2,f       ;Ciclo 2
        goto    dels_3

        decfsz  CntDel3,f       ;Ciclo 3 ripete ritardo 100ms
        goto    dels_2
        return


;-----------------------------------------------------
;Effettua ritardi da 1ms a 100ms (con interrupt abilitati)
;Con gli interrupt non abilitati durano circa 3 volte di meno

; Usate: CntDel1, CntDel2

deli_100ms                      ;Ritardo 100ms (100 * 1ms)
        movlw   D'100'
        goto    deli_1

deli_10ms                       ;Ritardo 10ms (10 * 1ms)
        movlw   D'10'
        goto    deli_1

deli_1ms                        ;Ritardo 1ms (1 * 1ms)
        movlw   D'1'
deli_1  movwf   CntDel2

deli_2  movlw   D'120'          ;Ritardo 1ms con irq on (tempo reale / 3 circa)
        movwf   CntDel1
deli_3  clrwdt                  ;[(6 * 120) / 2] * 3 = 1080 us --> 1ms circa
        nop
        nop
        decfsz  CntDel1,f       ;Ciclo 1
        goto    deli_3

        decfsz  CntDel2,f       ;Ciclo 2 ripete ritardo 1ms
        goto    deli_2
        return

